/*    Copyright (C) 2003 Dave Ackley                         (-*- C++ -*-)

      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using namespace std;

#include "Object.h"
#include "World.h"
#include "Parms.h"

Object::_vOp Object::_objects;
size_t Object::_time = 0;
size_t Object::_fishcount = 0;
size_t Object::_sharkcount = 0;
size_t Object::_dolphincount = 0;

Object::Object()
  : _x(~0), _y(~0), _breed(0), _starve(0), _idx(-1)
{
  _idx = _objects.size();
  _objects.push_back(this);
}

Object::~Object()
{
  if (_idx >= 0) {
    _objects[_idx] = 0;
  }
}

void Object::UpdateAll(World & w)
{

  // Compact out deaders
  static _vOp temp;

  temp.resize(0);
  for (size_t i = 0; i < _objects.size(); ++i) {
    if (_objects[i])
      temp.push_back(_objects[i]);
  }
  _objects = temp;

  // Shuffle the objects
  size_t max = _objects.size();

  for (size_t i = 0; i < max; ++i) {
    size_t other = random()%max;
    Object * temp = _objects[i];
    _objects[i] = _objects[other];
    _objects[other] = temp;
    
    _objects[i]->_idx = i; 
    _objects[other]->_idx = other; 
  }

  // Iterate over everybody
  for (size_t i = max; i > 0; ) {
    if (Object * o = _objects[--i])
      o->Update(w);
  }

  // Bump the clock
  ++_time;
}


bool Fish::Update(World & w)
{
  size_t ex = 0, ey = 0;        // Initialize to avoid compiler warning
  size_t count = 0;

  size_t oldx = _x, oldy = _y;

  for (size_t dir = World::NORTH; dir <= World::WEST; ++dir) {

    size_t cx = oldx, cy = oldy;
    w.GetXY(cx,cy,dir);

    if (w.Get(cx,cy) == 0) {

      if (random() % ++count == 0) {
        ex = cx;
        ey = cy;
      }
    }
  }

  if (count > 0) {
    w.Put(oldx, oldy, 0);
    w.Put(ex, ey, this);
    
    if (GetBreed() == 0) {

      w.Put(oldx, oldy, new Fish());
      ResetBreed();

    } else DecrBreed();

  } else {
    DecrBreed();
  }

  return true;
}

bool Shark::Update(World & w)
{
  size_t ex = 0, ey = 0;       // Initialize to avoid compiler warning
  size_t ecount = 0;
  size_t fx = 0, fy = 0;       // Initialize to avoid compiler warning
  size_t fcount = 0;

  size_t oldx = _x, oldy = _y;

  if (GetStarve() == 0) {
    w.Put(oldx, oldy, 0);

    delete this;
    return false;
  }

  DecrStarve();
  DecrBreed();

  for (size_t dir = World::NORTH; dir <= World::WEST; ++dir) {
    size_t cx = oldx, cy = oldy;
    w.GetXY(cx,cy,dir);

    Object * o = w.Get(cx,cy);
    if (o == 0) {

      if (random() % ++ecount == 0) {
        ex = cx;
        ey = cy;
      }
    } else if (o->IsFish()) {

      if (random() % ++fcount == 0) {
        fx = cx;
        fy = cy;
      }
    }
  }

  bool moved = false;

  if (fcount > 0) {             // Eating fish at fx,fy
    ResetStarve();

    Object *f = w.Get(fx,fy);

    w.Put(oldx,oldy, 0);
    w.Put(fx,fy,this);
    delete f;
    moved = true;

  } else if (ecount > 0) {      // Moving to empty at ex,ey

    w.Put(ex,ey, this);
    w.Put(oldx, oldy, 0);
    moved = true;
  } 

  if (moved && GetBreed() == 0) {
    w.Put(oldx, oldy, new Shark());
    ResetBreed();
  }

  return true;

}


bool Dolphin::Update(World & w)
{
  size_t ex = 0, ey = 0;       // Initialize to avoid compiler warning
  size_t ecount = 0;
  size_t fx = 0, fy = 0;       // Initialize to avoid compiler warning
  size_t fcount = 0;

  size_t oldx = _x, oldy = _y;

  if (GetStarve() == 0) {
    w.Put(oldx, oldy, 0);

    delete this;
    return false;
  }

  DecrStarve();
  DecrBreed();

  for (size_t dir = World::NORTH; dir <= World::WEST; ++dir) {
    size_t cx = oldx, cy = oldy;
    w.GetXY(cx,cy,dir);

    Object * o = w.Get(cx,cy);
    if (o == 0) {

      if (random() % ++ecount == 0) {
        ex = cx;
        ey = cy;
      }
    } else if (o->IsFish()||o->IsShark()) {

      if (random() % ++fcount == 0) {
        fx = cx;
        fy = cy;
      }
    }
  }

  bool moved = false;

  if (fcount > 0) {             // Eating fish at fx,fy
    ResetStarve();

    Object *f = w.Get(fx,fy);

    w.Put(oldx,oldy, 0);
    w.Put(fx,fy,this);
    delete f;
    moved = true;

  } else if (ecount > 0) {      // Moving to empty at ex,ey

    w.Put(ex,ey, this);
    w.Put(oldx, oldy, 0);
    moved = true;
  } 

  if (moved && GetBreed() == 0) {
    w.Put(oldx, oldy, new Dolphin());
    ResetBreed();
  }

  return true;

}
