/*    Copyright (C) 2003 Dave Ackley                         (-*- C++ -*-)

      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef OBJECT_H
#define OBJECT_H

// Ackley for CS591 F'2003

#include <assert.h>
#include <vector>
#include "Parms.h"

class World;                      // Forward

class Object {
public:

  Object() ;
  virtual ~Object() ;

  virtual bool Update(World &) = 0;
  virtual bool IsFish() const { return false; }
  virtual bool IsShark() const { return false; }

  size_t GetBreed() const { return _breed; }
  void DecrBreed() { if (_breed) --_breed; }
  virtual void ResetBreed() = 0;

  u_char GetStarve() const { return _starve; }
  void DecrStarve() { if (_starve) --_starve; }
  virtual void ResetStarve() = 0;

  virtual char Code() const = 0;

  void SetXY(size_t x, size_t y) { _x = x; _y = y; }
  void GetXY(size_t & x, size_t & y) { x = _x; y = _y; }

  static void UpdateAll(World & ) ;
  static size_t GetFishCount() { return _fishcount; }
  static size_t GetDolphinCount() { return _dolphincount; }
  static size_t GetSharkCount() { return _sharkcount; }
  static size_t GetTime() { return _time; }

protected:

  static size_t _time;
  static size_t _fishcount;
  static size_t _dolphincount;
  static size_t _sharkcount;

  size_t _x;
  size_t _y;
  size_t _breed;
  size_t _starve;

private:
  typedef vector<Object *> _vOp; 
  static _vOp _objects;
  int _idx;

} ;

class Fish : public Object {
public:

  Fish() { ++_fishcount; ResetBreed(); }
  ~Fish() { assert(_fishcount); --_fishcount; }

  bool IsFish() const { return true; }
  char Code() const { return '.'; }
  bool Update(World &) ;
  void ResetBreed() { _breed = Parms::Global().GetFishBreed(); }
  void ResetStarve() { assert(0); }

private:
};

class Shark : public Object {
public:

  Shark() {
    ++_sharkcount;
    ResetStarve();
    ResetBreed();
  }
  ~Shark() { assert(_sharkcount); --_sharkcount; }

  bool IsShark() const { return true; }
  char Code() const { return '*'; }
  bool Update(World &) ;
  void ResetBreed() { _breed = Parms::Global().GetSharkBreed(); }
  void ResetStarve() { _starve = Parms::Global().GetSharkStarve(); }

private:
};

class Dolphin : public Object {
public:
                                                                                
  Dolphin() {
    ++_dolphincount;
    ResetStarve();
    ResetBreed();
  }
  ~Dolphin() { assert(_dolphincount); --_dolphincount; }
                                                                                
  char Code() const { return ','; }
  bool Update(World &) ;
  void ResetBreed() { _breed = Parms::Global().GetDolphinBreed(); }
  void ResetStarve() { _starve = Parms::Global().GetDolphinStarve(); }
                                                                                
private:
};


#endif
