/*    Copyright (C) 2003 Dave Ackley                         (-*- C++ -*-)

      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using namespace std;

#include "main.h"

#include <iostream>

int main(int argc, char ** argv)
{
  // Process args, establish globally accessible parameters
  Parms p;
  if (!p.ProcessArgs(argc,argv)) {
    cerr << "Parameter processing failed" << endl;
    return 1;
  }
  p.SetGlobal();

  // Sanity check sizes
  size_t initfish = p.GetInitFish();
  size_t initshark = p.GetInitShark();
  size_t initdolphin = p.GetInitDolphin();
  size_t width = p.GetWidth();
  size_t height = p.GetHeight();

  if (initfish + initshark + initdolphin == 0) {
    cerr << "Need some fish, sharks and or dolphins" << endl;
    return 2;
  }

  if (width * height < initfish + initshark + initdolphin) {
    cerr << "World too small for initfish+initshark+initdolphin" << endl;
    return 3;
  }

  // Default the seed
  if (p.GetSeed() == 0) 
    p.SetSeed(time(0));

  // Echo run parameters
  cerr
    << "-f " << p.GetFlags() <<  ' '
    << "-s " << p.GetSeed() <<  ' '
    << "-w " << p.GetWidth() << ' '
    << "-h " << p.GetHeight() << ' '
    << "-if " << p.GetInitFish() << ' '
    << "-is " << p.GetInitShark() << ' '
    << "-id " << p.GetInitDolphin() << ' '
    << "-fb " << static_cast<size_t>(p.GetFishBreed()) << ' '
    << "-sb " << static_cast<size_t>(p.GetSharkBreed()) << ' '
    << "-ss " << static_cast<size_t>(p.GetSharkStarve()) << ' '
    << "-db " << static_cast<size_t>(p.GetDolphinBreed()) << ' '
    << "-ds " << static_cast<size_t>(p.GetDolphinStarve()) << ' '

    << endl;

  // Init seed
  srandom(p.GetSeed());


  // Build world
  World m;
  m.Resize(width,height);

  // Load up objects
  for (size_t x = 0; x < width; ++x) {
    for (size_t y = 0; y < height; ++y) {
      if (initfish) {
        m.Put(x,y,new Fish);
        --initfish;
      } else if (initshark) {
        m.Put(x,y,new Shark);
        --initshark;
      } else if (initdolphin) {
	m.Put(x,y,new Dolphin);
	--initdolphin;
      } else m.Put(x,y,0);
    }
  }

  // Shuffle them
  for (size_t x = 0; x < width; ++x) {
    for (size_t y = 0; y < height; ++y) {
      size_t rx = random()%width;
      size_t ry = random()%height;

      Object * tmp = m.Get(x,y);
      m.Put(x,y,m.Get(rx,ry));
      m.Put(rx,ry,tmp);
    }
  }

  // Dump initial world 
  m.Dump(cout);

  bool forever = p.GetFlag(Parms::DONTSTOP);

  // Main loop
  while (1) {
    Object::UpdateAll(m);
    m.Dump(cout);
    if (!forever && (Object::GetFishCount()==0 || Object::GetSharkCount()==0 || Object::GetDolphinCount()==0 ))
      break;
  }

  return 0;
}
